@@ -244,11 +244,10 @@ int main(int argc, char *argv[])
 	if (version_only)
 		exit(EXIT_FAILURE);
 
-	if (argc < 2)
+	if (argc - optind != 1)
 		usage();
 
-	memset(ui.dev_name, 0, sizeof(ui.dev_name));
-	snprintf(ui.dev_name, sizeof(ui.dev_name), "%s", argv[1]);
+	ui.dev_name = argv[1];
 
 	ret = exfat_get_blk_dev_info(&ui, &bd);
 	if (ret < 0)
